package de.fhdw.wtf.common.ast;

import de.fhdw.wtf.common.token.Token;

/**
 * This class represents a syntax object. Every part of the abstract syntax tree will have this class. This class has
 * two {@link Token}. The first and last token of a parsed part. When an object is a synthetic object, both tokens are
 * the {@link de.fhdw.wtf.common.token.DummyToken}.
 */
public abstract class SyntaxObject implements SyntaxObjectInterface {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 7689824529366354331L;
	
	/**
	 * First-Token.
	 */
	private Token firstToken;
	
	/**
	 * Last-Token.
	 */
	private Token lastToken;
	
	/**
	 * Constructor of {@link SyntaxObject}.
	 * 
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 */
	protected SyntaxObject(final Token firstToken, final Token lastToken) {
		this.setFirstToken(firstToken);
		this.setLastToken(lastToken);
	}
	
	// //////////// Projections //////////////////
	
	@Override
	public Token getFirstToken() {
		return this.firstToken;
	}
	
	@Override
	public void setFirstToken(final Token firstToken) {
		this.firstToken = firstToken;
	}
	
	@Override
	public Token getLastToken() {
		return this.lastToken;
	}
	
	@Override
	public void setLastToken(final Token lastToken) {
		this.lastToken = lastToken;
	}
	
}
