package de.fhdw.wtf.common.ast.type;

import java.util.Collection;

import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.ast.visitor.AtomicTypeVisitor;
import de.fhdw.wtf.common.ast.visitor.AtomicTypeVisitorException;
import de.fhdw.wtf.common.ast.visitor.AtomicTypeVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.AtomicTypeVisitorReturnException;
import de.fhdw.wtf.common.ast.visitor.TypeVisitor;
import de.fhdw.wtf.common.ast.visitor.TypeVisitorException;
import de.fhdw.wtf.common.ast.visitor.TypeVisitorReturn;
import de.fhdw.wtf.common.ast.visitor.TypeVisitorReturnException;
import de.fhdw.wtf.common.exception.parser.SumIsAnythingException;
import de.fhdw.wtf.common.token.Token;

/**
 * This class represents a AtomicType. A AtomicType can be a {@link ClassType} or {@link BaseType}.
 * 
 */
public abstract class AtomicType extends RealType {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = 4813520984560358830L;
	
	/**
	 * The name of the AtomicType.
	 */
	private final Name typeName;
	
	/**
	 * Constructor of {@link AtomicType}.
	 * 
	 * @param typeName
	 *            name of the atomicType
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 */
	protected AtomicType(final Name typeName, final Token firstToken, final Token lastToken) {
		super(firstToken, lastToken);
		this.typeName = typeName;
	}
	
	/**
	 * Constructor of {@link AtomicType}.
	 * 
	 * @param typeName
	 *            name of the atomicType
	 * @param firstToken
	 *            firstToken
	 * @param lastToken
	 *            lastToken
	 * @param superTypes
	 *            superTypes
	 */
	protected AtomicType(final Name typeName,
			final Token firstToken,
			final Token lastToken,
			final Collection<Type> superTypes) {
		super(firstToken, lastToken, superTypes);
		this.typeName = typeName;
	}
	
	@Override
	public boolean equals(final Object o) {
		if (o instanceof TypeProxy) {
			return o.equals(this);
		}
		if (o instanceof AtomicType) {
			final AtomicType simpleType = (AtomicType) o;
			return this.typeName.equals(simpleType.getTypeName());
		}
		return false;
	}
	
	@Override
	public int hashCode() {
		return this.typeName.hashCode();
	}
	
	/**
	 * Get the name of the Atomic-Type.
	 * 
	 * @return Name
	 */
	public Name getTypeName() {
		return this.typeName;
	}
	
	@Override
	public void accept(final TypeVisitor v) {
		v.handle(this);
	}
	
	@Override
	public <X> X accept(final TypeVisitorReturn<X> v) {
		return v.handle(this);
	}
	
	@Override
	public <X extends Exception> void accept(final TypeVisitorException<X> v) throws X {
		v.handle(this);
	}
	
	@Override
	public <X, Y extends Exception> X accept(final TypeVisitorReturnException<X, Y> v) throws Y {
		return v.handle(this);
	}
	
	/**
	 * Accept-Methode Visitor-Pattern.
	 * 
	 * @param v
	 *            Visitor
	 */
	public abstract void accept(AtomicTypeVisitor v);
	
	/**
	 * Accept-Methode Visitor-Pattern.
	 * 
	 * @param v
	 *            Visitor
	 * @param <Y>
	 *            Exception-Type
	 * @throws Y
	 *             Exception
	 */
	public abstract <Y extends Exception> void accept(AtomicTypeVisitorException<Y> v) throws Y;
	
	/**
	 * Accept-Methode Visitor-Pattern.
	 * 
	 * @param v
	 *            Visitor
	 * @param <X>
	 *            ReturnType
	 * @return X
	 */
	public abstract <X> X accept(AtomicTypeVisitorReturn<X> v);
	
	/**
	 * Accept-Methode Visitor-Pattern.
	 * 
	 * @param v
	 *            Visitor
	 * @param <X>
	 *            ExceptionType
	 * @param <Y>
	 *            ReturnType
	 * @return X
	 * @throws Y
	 *             Exception
	 */
	public abstract <X, Y extends Exception> X accept(AtomicTypeVisitorReturnException<X, Y> v) throws Y;
	
	@Override
	public Type standardize() throws SumIsAnythingException {
		return this;
	}
	
	@Override
	public boolean isThrownType() {
		return false;
	}
}
