package de.fhdw.wtf.common.ast.visitor;

import de.fhdw.wtf.common.ast.ConstructorByTypeAndSignatureState;
import de.fhdw.wtf.common.ast.ConstructorByReferenceState;
import de.fhdw.wtf.common.ast.ConstructorInvalidState;

/**
 * Return-Exception-Visitor for {@link de.fhdw.wtf.common.ast.ConstructorReferenceState}.
 * 
 * @param <X>
 *            return type of this visitor
 * @param <Y>
 *            ExceptionType
 */
public interface ConstructorReferenceStateVisitorReturnException<X, Y extends Exception> {
	
	/**
	 * Handle operations for the type {@link ConstructorByTypeAndSignatureState}.
	 * 
	 * @param byName
	 *            {@link ConstructorByTypeAndSignatureState}
	 * @return X
	 * @throws Y
	 *             Y
	 */
	X handle(ConstructorByTypeAndSignatureState byName) throws Y;
	
	/**
	 * Handle operations for the type {@link ConstructorInvalidState}.
	 * 
	 * @param invalid
	 *            {@link ConstructorInvalidState}
	 * @return X
	 * @throws Y
	 *             Y
	 */
	X handle(ConstructorInvalidState invalid) throws Y;
	
	/**
	 * Handle operations for the type {@link ConstructorByReferenceState}.
	 * 
	 * @param byReference
	 *            {@link ConstructorByReferenceState}
	 * @return X
	 * @throws Y
	 *             Y
	 */
	X handle(ConstructorByReferenceState byReference) throws Y;
}
