package de.fhdw.wtf.common.exception.editor;

import de.fhdw.wtf.common.token.Position;

/**
 * Markable check exception.
 */
public class MarkableCheckException extends CheckException implements EditorMarkable {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -1240762509579304749L;
	
	/**
	 * Description.
	 */
	private final String description;
	
	/**
	 * Start-Position.
	 */
	private final Position startPos;
	
	/**
	 * End-Position.
	 */
	private final Position endPos;
	
	/**
	 * Marker.
	 */
	private final EditorMarker marker;
	
	/**
	 * Constructor of {@link MarkableCheckException}.
	 * 
	 * @param startPos
	 *            Start-Position
	 * @param endPos
	 *            End-Position
	 * @param inner
	 *            origin exception
	 */
	public MarkableCheckException(final Position startPos,
			final Position endPos,
			final Exception inner,
			final EditorMarker marker) {
		super(inner);
		this.startPos = startPos;
		this.endPos = endPos;
		this.description = inner.getMessage();
		this.marker = marker;
	}
	
	@Override
	public Position getStartPosition() {
		return this.startPos;
	}
	
	@Override
	public String getDescription() {
		return this.description;
	}
	
	@Override
	public void accept(final CheckExceptionVisitor v) {
		v.handle(this);
	}
	
	@Override
	public Position getEndPos() {
		return this.endPos;
	}
	
	@Override
	public String getMessage() {
		return this.description;
	}
	
	@Override
	public EditorMarker getMarker() {
		return this.marker;
	}
}
