package de.fhdw.wtf.common.exception.referencer;

import de.fhdw.wtf.common.ast.Name;
import de.fhdw.wtf.common.token.Token;

/**
 * A Method of the {@link de.fhdw.wtf.walker.tasks.TypeReferencer} throws it, if it can not find a class for a name.
 * 
 */
public final class InvalidTypeReferenceException extends ReferencerException {
	
	/**
	 * generated.
	 * 
	 */
	private static final long serialVersionUID = -8216115954693202252L;
	
	/**
	 * Name which not found as class.
	 */
	private final Name name;
	
	/**
	 * Constructor of {@link InvalidTypeReferenceException}.
	 * 
	 * @param message
	 *            message
	 * @param name
	 *            name which not found as class
	 * @param token
	 *            token
	 */
	private InvalidTypeReferenceException(final String message, final Name name, final Token token) {
		super(token, message + name.toString());
		this.name = name;
	}
	
	/**
	 * Creates a {@link InvalidTypeReferenceException}-Onject.
	 * 
	 * @param name
	 *            name which not found as class
	 * @param token
	 *            token
	 * @return The {@link InvalidTypeReferenceException}-Onject.
	 */
	public static InvalidTypeReferenceException create(final Name name, final Token token) {
		return new InvalidTypeReferenceException(
				de.fhdw.wtf.common.constants.referencer.ExceptionConstans.INVALID_TYPE_REFERENCE_EXCEPTION_MESSAGE,
				name, token);
	}
	
	/**
	 * Returns the name which not found a class.
	 * 
	 * @return name
	 */
	public Name getName() {
		return this.name;
	}
	
}
