package de.fhdw.wtf.common.exception.walker;

import de.fhdw.wtf.common.ast.Attribute;
import de.fhdw.wtf.common.exception.editor.EditorMarkable;
import de.fhdw.wtf.common.exception.editor.EditorMarker;
import de.fhdw.wtf.common.token.Position;

/**
 * Will be thrown if the {@link de.fhdw.wtf.walker.tasks.DoubleAttributenameCheck} found twice attributes with the same
 * name in a class and his inheritance-structur.
 * 
 */
public final class DoubleAttributenameException extends TaskException implements EditorMarkable {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -6608366879366425685L;
	
	/**
	 * One Attribute with the same name as another attribute.
	 */
	private final Attribute attribute;
	
	/**
	 * Constructor of {@link DoubleAttributenameException}.
	 * 
	 * @param attribute
	 *            attribute
	 */
	private DoubleAttributenameException(final Attribute attribute) {
		super(de.fhdw.wtf.common.constants.walker.ExceptionConstants.DOUBLE_ATTRIBUTENAME_MESSAGE + attribute.getName());
		this.attribute = attribute;
	}
	
	/**
	 * Creates a {@link DoubleAttributenameException}-Object.
	 * 
	 * @param a
	 *            attribute
	 * @return The {@link DoubleAttributenameException}-Object.
	 */
	public static DoubleAttributenameException create(final Attribute a) {
		return new DoubleAttributenameException(a);
	}
	
	@Override
	public Position getStartPosition() {
		return this.attribute.getFirstToken().getPosition();
	}
	
	@Override
	public String getDescription() {
		return this.getMessage();
	}
	
	@Override
	public Position getEndPos() {
		return this.attribute.getLastToken().getPosition();
	}
	
	@Override
	public EditorMarker getMarker() {
		return EditorMarker.DOUBLE_ATTRIBUTENAME_MARKER;
	}
	
}
