package de.fhdw.wtf.common.stream;

import de.fhdw.wtf.common.token.Token;

/**
 * Definition of a sequence of Token.
 */
public interface TokenStream {
	
	/**
	 * Adds the {@link Token} <code>t</code> to the end of the stream.
	 * 
	 * @param t
	 *            {@link Token} to be added.
	 */
	void add(Token t);
	
	/**
	 * Returns the next {@link Token} element from the stream. <br>
	 * <br>
	 * <code>
	 * TokenStream stream = new ...; <br>
	 * stream.next() // first element <br>
	 * stream.next() // second element <br>
	 * </code> and so on. <br>
	 * 
	 * @return the next {@link Token}.
	 */
	Token next();
	
	/**
	 * Tests if there is at least one more {@link Token} at the Stream.
	 * 
	 * @return <code>true</code> if there are {@link Token}s left at the Stream. <br>
	 *         <code>false</code> if the Stream is empty.
	 */
	boolean hasNext();
	
	/**
	 * Removes the first Token in the stream.
	 * 
	 * @return Returns true if an element is removed, false otherwise.
	 */
	boolean removeFirst();
	
	/**
	 * Returns the next {@link Token} element without removing it from the stream.
	 * 
	 * @return the next {@link Token}.
	 */
	Token peek();
	
	/**
	 * Return a Copy of the {@link TokenStream}.
	 * 
	 * @return copied Tokenstream
	 */
	TokenStream copy();
	
}
