package de.fhdw.wtf.common.token.symbols;

import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.common.token.Token;

/**
 * Represents the arrow symbol '->' as a token.
 */
public final class ArrowToken extends SymbolToken {
	
	/**
	 * generated.
	 */
	private static final long serialVersionUID = -8968302893086346460L;
	
	/**
	 * Defines constant for hashCodeTemplate().
	 */
	private static final int HASHCODE_TEMPLATE_CONSTANT = 97834579;
	
	/**
	 * Private constructor for {@link ArrowToken}. Use create(...)-Factory instead!
	 * 
	 * @param position
	 *            represents the start position of the symbol '->' in the original input.
	 */
	private ArrowToken(final Position position) {
		super(position, SymbolConstants.ARROW);
	}
	
	/**
	 * Factory method for {@link ArrowToken}.
	 * 
	 * @param position
	 *            represents the start position of this {@link SymbolToken} in the original input.
	 * @return a new instance of this {@link SymbolToken}.
	 */
	public static ArrowToken create(final Position position) {
		return new ArrowToken(position);
	}
	
	@Override
	public boolean isArrowToken() {
		return true;
	}
	
	@Override
	protected boolean equalsTemplate(final Token t) {
		return t.isArrowToken();
	}
	
	@Override
	public String stringRepresentation() {
		return SymbolConstants.ARROW;
	}
	
	@Override
	protected int hashCodeTemplate() {
		return HASHCODE_TEMPLATE_CONSTANT;
	}
	
}
