package de.fhdw.wtf.dsl.scanner.modelScanner.state;

import de.fhdw.wtf.common.stream.TokenStream;
import de.fhdw.wtf.common.token.Position;
import de.fhdw.wtf.dsl.scanner.common.ScannerState;

/**
 * State for processing multi-char elements.
 */
public abstract class MultiCharState implements ScannerState {
	
	/**
	 * Beginning characters which have not been reflected in a token.
	 */
	private String currentInput;
	
	/**
	 * Position of the first input character in the original input stream.
	 */
	private final Position firstPosition;
	
	/**
	 * Constructor.
	 * 
	 * @param currentInput
	 *            initial input.
	 * @param firstPosition
	 *            of the first char.
	 */
	protected MultiCharState(final String currentInput, final Position firstPosition) {
		this.setCurrentInput(currentInput);
		this.firstPosition = firstPosition;
	}
	
	@Override
	public void process(final char input, final TokenStream outputStream, final Position position) {
		this.append(input);
	}
	
	/**
	 * Appends input at the end of the {@link MultiCharState#currentInput}.
	 * 
	 * @param input
	 *            character to be appended to the currentInput field
	 */
	protected void append(final char input) {
		this.setCurrentInput(this.getCurrentInput() + input);
	}
	
	/**
	 * Setter for {@link MultiCharState#currentInput}.
	 * 
	 * @param newCurrentInput
	 *            new value of currentInput
	 */
	protected void setCurrentInput(final String newCurrentInput) {
		this.currentInput = newCurrentInput;
	}
	
	/**
	 * Getter for {@link MultiCharState#currentInput}<br>
	 * witch holds the by now processed <code>String</code>.
	 * 
	 * @return {@link MultiCharState#currentInput}
	 */
	protected String getCurrentInput() {
		return this.currentInput;
	}
	
	/**
	 * Getter for the position of the first character scanned by this state.
	 * 
	 * @return first character position
	 */
	public Position getFirstPosition() {
		return this.firstPosition;
	}
}
