package de.fhdw.wtf.tooling.wizard;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map.Entry;
import java.util.Properties;

import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IClasspathAttribute;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.internal.core.ClasspathAttribute;
import org.eclipse.jdt.internal.core.ClasspathEntry;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;

import com.google.gdt.eclipse.maven.MavenUtils;

@SuppressWarnings("restriction")
public class ProjectWizard extends Wizard implements INewWizard {
	
	public static final String BUILDER_ID = "de.fhdw.wtf.tools.builders.WTFBuilder";
	private static final String APPLICATION_CONFIG_CONTENT =
			"# Central Application Configuration File\n#\n# Provides information about Persistence Provider and other central Information needed for a Launch of the Application\n#\ndatabase=OracleDatabaseManagerFactory";
	private WTFWizard pageOne;
	
	public ProjectWizard() {
		this.setWindowTitle("WTF Project");
	}
	
	@Override
	public void addPages() {
		super.addPages();
		this.pageOne = new WTFWizard("New WTF Project Wizard");
		this.pageOne.setTitle("New WTF Project");
		this.pageOne.setDescription("Create a new Web Transactions Framework Project");
		this.addPage(this.pageOne);
	}
	
	@Override
	public void init(final IWorkbench workbench, final IStructuredSelection selection) {
		// nothing to do
	}
	
	@Override
	public boolean performFinish() {
		final IWorkspaceRoot workspaceRoot = ResourcesPlugin.getWorkspace().getRoot();
		final IProject project = workspaceRoot.getProject(this.pageOne.getProjectName());
		try {
			project.create(null);
			project.open(null);
			final IProjectDescription description = project.getDescription();
			description.setNatureIds(new String[] { "org.eclipse.ajdt.ui.ajnature", JavaCore.NATURE_ID,
					MavenUtils.MAVEN2_NATURE_ID });
			final ICommand builderCommand = description.newCommand();
			builderCommand.setBuilderName(BUILDER_ID);
			final List<ICommand> commands = new ArrayList<>();
			commands.addAll(Arrays.asList(description.getBuildSpec()));
			commands.add(builderCommand);
			description.setBuildSpec(commands.toArray(new ICommand[commands.size()]));
			project.setDescription(description, null);
			final IJavaProject javaProject = JavaCore.create(project);
			final IClasspathEntry[] buildPath = this.createClassPath(project);
			javaProject.setRawClasspath(buildPath, project.getFullPath().append("target"), null);
			final IFolder javaFolder = this.createFolders(project);
			javaFolder.create(true, true, null);
			final IFolder folderResources = project.getFolder("src/main/resources");
			folderResources.create(true, true, null);
			this.createPackages(javaProject, javaFolder);
			final IFile pom = project.getFile("pom.xml");
			pom.create(
					new ByteArrayInputStream(this.getPomContents(
							project.getName(),
							this.pageOne.getTopLevelPackageName())),
					true,
					null);
			final IFile launch = project.getFile("generate_" + project.getName() + ".launch");
			launch.create(
					new ByteArrayInputStream(this.getLaunchConfigContents(project.getFullPath().toString())),
					true,
					null);
			final IFile applicationStarterFile =
					javaFolder.getFile("core/application/GeneratedApplicationStarter.java");
			final IPackageFragmentRoot fragmentRootResources = javaProject.getPackageFragmentRoot(folderResources);
			fragmentRootResources.createPackageFragment("models", true, null);
			fragmentRootResources.createPackageFragment("config", true, null);
			this.createFilesWithContent(folderResources, applicationStarterFile);
		} catch (final CoreException e) {
			e.printStackTrace();
		}
		return true;
	}
	
	private void createFilesWithContent(final IFolder folderResources, final IFile applicationStarterFile)
			throws CoreException {
		final IFile file = folderResources.getFile("models/" + this.pageOne.getModelFileName() + ".wtf");
		final IFile fileOracleProps = folderResources.getFile("config/oracledb.properties");
		final IFile fileApplicationProps = folderResources.getFile("config/application.properties");
		InputStream stream = new ByteArrayInputStream(this.getModelFileContents(this.pageOne.getTopLevelPackageName()));
		file.create(stream, true, null);
		stream = new ByteArrayInputStream(this.getOracleDbPropertiesAsString(this.pageOne.getOracleProps()).getBytes());
		fileOracleProps.create(stream, true, null);
		final String input = APPLICATION_CONFIG_CONTENT;
		stream = new ByteArrayInputStream(input.getBytes());
		fileApplicationProps.create(stream, true, null);
		stream = new ByteArrayInputStream(this.getApplicationStarterContents());
		applicationStarterFile.create(stream, true, null);
	}
	
	private void createPackages(final IJavaProject javaProject, final IFolder javaFolder) throws JavaModelException {
		final IPackageFragmentRoot fragementRootJava = javaProject.getPackageFragmentRoot(javaFolder);
		fragementRootJava.createPackageFragment("generated.model", true, null);
		fragementRootJava.createPackageFragment("core.application", true, null);
	}
	
	private byte[] getModelFileContents(final String groupName) {
		final StringBuilder result = new StringBuilder();
		final String[] groupTokens = groupName.split(">");
		int j = 0;
		for (int i = 0; i < groupTokens.length; i++) {
			result.append(this.getTabs(i) + groupTokens[i] + " : group = [\n");
			j = i;
		}
		result.append(this.getTabs(j + 1) + "# Insert domainmodels here #\n");
		for (int i = j; i >= 0; i--) {
			result.append(this.getTabs(i) + "];\n");
		}
		return result.toString().getBytes();
	}
	
	private String getTabs(final int amount) {
		final StringBuilder result = new StringBuilder();
		for (int i = 0; i < amount; i++) {
			result.append("\t");
		}
		return result.toString();
	}
	
	private IFolder createFolders(final IProject project) throws CoreException {
		project.getFolder("src").create(true, true, null);
		project.getFolder("src/main").create(true, true, null);
		project.getFolder("src/test").create(true, true, null);
		project.getFolder("src/test/java").create(true, true, null);
		project.getFolder("src/test/resources").create(true, true, null);
		project.getFolder("src/main/webapp").create(true, true, null);
		final IFolder javaFolder = project.getFolder("src/main/java");
		return javaFolder;
	}
	
	private IClasspathEntry[] createClassPath(final IProject project) {
		// Path wtfPersistenceJar =null;
		// Path wtfContextJar = null;
		// Path ojdbc6Jar = null;
		// Enumeration<URL> e = Platform.getBundle("WTFProjectWizard").findEntries("resources", "*.jar", false);
		// while (e.hasMoreElements()) {
		// URL current = e.nextElement();
		// if (current.toString().endsWith("wtf-context.jar")) {
		// wtfContextJar = new Path(Platform.asLocalURL(current).getPath());
		// }
		// if (current.toString().endsWith("wtf-persistence.jar")) {
		// wtfPersistenceJar = new Path(Platform.asLocalURL(current).getPath());
		// }
		// if (current.toString().endsWith("ojdbc6.jar")) {
		// ojdbc6Jar = new Path(Platform.asLocalURL(current).getPath());
		// }
		// }
		// IClasspathAttribute[] classPathAttributes = new IClasspathAttribute[1];
		final IClasspathEntry mavenDependencies =
				JavaCore.newContainerEntry(
						new Path("org.eclipse.m2e.MAVEN2_CLASSPATH_CONTAINER"),
						ClasspathEntry.NO_ACCESS_RULES,
						new IClasspathAttribute[] { new ClasspathAttribute("maven.pomderived", "true") },
						false);
		// classPathAttributes[0] = JavaCore.newClasspathAttribute("org.eclipse.ajdt.inpath",
		// "org.eclipse.ajdt.inpath");
		// IClasspathEntry contexClasspathEntries = JavaCore.newLibraryEntry(wtfContextJar,null,null,new
		// IAccessRule[0],classPathAttributes,false);
		final IClasspathEntry[] buildPath =
				{ JavaCore.newSourceEntry(project.getFullPath().append("src/main/java")),
						JavaCore.newSourceEntry(project.getFullPath().append("src/main/resources")),
						JavaCore.newSourceEntry(project.getFullPath().append("src/test/java")),
						JavaCore.newSourceEntry(project.getFullPath().append("src/test/resources")),
						// contexClasspathEntries,
						// JavaCore.newLibraryEntry(wtfPersistenceJar,null,null),
						// JavaCore.newLibraryEntry(ojdbc6Jar,null,null),
						JavaRuntime.getDefaultJREContainerEntry(), mavenDependencies
				// new AspectJRTContainer().getClasspathEntries()[0]
				};
		return buildPath;
	}
	
	private byte[] getApplicationStarterContents() {
		final StringBuffer result = new StringBuffer();
		result.append("package core.application;\n\n");
		result.append("import de.fhdw.wtf.context.core.ApplicationStarter;\n");
		result.append("import de.fhdw.wtf.context.core.ObjectFactoryProvider;\n");
		result.append("import de.fhdw.wtf.persistence.exception.TypeOrAssociationNotFoundException;\n");
		result.append("import de.fhdw.wtf.persistence.facade.TypeManager;\n\n");
		result.append("public class " + "GeneratedApplicationStarter " + "extends ApplicationStarter {\n\n");
		result.append("\t@Override\n");
		result.append("\tprotected void registerTypeFactories(ObjectFactoryProvider factoryProvider,\n");
		result.append("\t		TypeManager typeManager) throws TypeOrAssociationNotFoundException {\n");
		result.append("\t}\n\n");
		result.append("\t@Override\n");
		result.append("\tprotected String getResourcesPathJava() {\n");
		result.append("\t	return \"src/main/resources/config\";\n");
		result.append("\t}\n\n");
		result.append("\t@Override\n");
		result.append("\tprotected String getResourcesPathServer() {\n");
		result.append("\t	return \"WEB-INF\";\n");
		result.append("\t}\n\n");
		result.append("\t@Override\n");
		result.append("\tprotected void registerActivities() {\n");
		result.append("\t	\n");
		result.append("\t}\n\n");
		result.append("\t@Override\n");
		result.append("\tprotected String getApplicationConfigFileName() {\n");
		result.append("\t	return \"application.properties\";\n");
		result.append("\t}\n\n");
		result.append("\t@Override\n");
		result.append("\tprotected String getDatabaseConfigFileName() {\n");
		result.append("\t	return \"oracledb.properties\";\n");
		result.append("\t}\n\n");
		result.append("\t@Override\n");
		result.append("\tprotected String getModelPrefix() {\n");
		result.append("\t	return \"generated.model\";\n");
		result.append("\t}\n}");
		return result.toString().getBytes();
	}
	
	private byte[] getPomContents(final String projectName, final String groupName) {
		final StringBuffer result = new StringBuffer();
		final String replacedGroupName = groupName.replace(">", ".");
		System.out.println(replacedGroupName);
		result.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		result.append("<project xmlns=\"http://maven.apache.org/POM/4.0.0\" \n");
		result.append("	xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" \n");
		result.append("	xsi:schemaLocation=\"http://maven.apache.org/POM/4.0.0 http://maven.apache.org/xsd/maven-4.0.0.xsd \">\n");
		result.append("<modelVersion>4.0.0</modelVersion>\n");
		result.append("<groupId>" + replacedGroupName + "</groupId>\n");
		result.append("<artifactId>" + projectName + "</artifactId>\n");
		result.append("<version>1.0-SNAPSHOT</version>\n");
		result.append("<packaging>jar</packaging>\n");
		result.append("<build>\n");
		result.append("\t<plugins>\n");
		result.append("\t\t<plugin>\n");
		result.append("\t\t\t<groupId>org.apache.maven.plugins</groupId>\n");
		result.append("\t\t\t<artifactId>maven-compiler-plugin</artifactId>\n");
		result.append("\t\t\t<version>3.3</version>\n");
		result.append("\t\t\t<configuration>\n");
		result.append("\t\t\t\t<source>1.7</source>\n");
		result.append("\t\t\t\t<target>1.7</target>\n");
		result.append("\t\t\t</configuration>\n");
		result.append("\t\t</plugin>\n");
		result.append("\t\n");
		result.append("\t\t<plugin>\n");
		result.append("\t\t\t<groupId>org.codehaus.mojo</groupId>\n");
		result.append("\t\t\t<artifactId>aspectj-maven-plugin</artifactId>\n");
		result.append("\t\t\t<version>1.8</version>\n");
		result.append("\t\t\t<configuration>\n");
		result.append("\t\t\t\t<verbose>true</verbose>\n");
		result.append("\t\t\t\t<showWeaveInfo>true</showWeaveInfo>\n");
		result.append("\t\t\t\t<complianceLevel>1.7</complianceLevel>\n");
		result.append("\t\t\t\t<source>1.7</source>\n");
		result.append("\t\t\t\t<target>1.7</target>\n");
		result.append("\t\t\t\t<aspectLibraries>\n");
		result.append("\t\t\t\t\t<aspectLibrary>\n");
		result.append("\t\t\t\t\t\t<groupId>de.fhdw.wtf</groupId>\n");
		result.append("\t\t\t\t\t\t<artifactId>de.fhdw.wtf.context</artifactId>\n");
		result.append("\t\t\t\t\t</aspectLibrary>\n");
		result.append("\t\t\t\t</aspectLibraries>\n");
		result.append("\t\t\t</configuration>\n");
		result.append("\t\t\t<executions>\n");
		result.append("\t\t\t\t<execution>\n");
		result.append("\t\t\t\t\t<goals>\n");
		result.append("\t\t\t\t\t\t<goal>compile</goal>\n");
		result.append("\t\t\t\t\t\t<goal>test-compile</goal>\n");
		result.append("\t\t\t\t\t</goals>\n");
		result.append("\t\t\t\t</execution>\n");
		result.append("\t\t\t</executions>\n");
		result.append("\t\t</plugin>\n");
		result.append("\t\t<plugin>\n");
		result.append("\t\t\t<groupId>de.fhdw.wtf</groupId>\n");
		result.append("\t\t\t<artifactId>wtf-maven-plugin</artifactId>\n");
		result.append("\t\t\t<version>1.0-SNAPSHOT</version>\n");
		result.append("\t\t</plugin>\n");
		result.append("\t</plugins>\n");
		result.append("</build>\n");
		result.append("<dependencies>\n");
		result.append("\t<dependency>\n");
		result.append("\t\t<groupId>junit</groupId>\n");
		result.append("\t\t<artifactId>junit</artifactId>\n");
		result.append("\t\t<version>4.12</version>\n");
		result.append("\t\t<scope>test</scope>\n");
		result.append("\t</dependency>\n");
		result.append("\t<dependency>\n");
		result.append("\t\t<groupId>de.fhdw.wtf</groupId>\n");
		result.append("\t\t<artifactId>de.fhdw.wtf.context</artifactId>\n");
		result.append("\t\t<version>1.0-SNAPSHOT</version>\n");
		result.append("\t</dependency>\n");
		result.append("</dependencies>\n");
		result.append("</project>\n");
		return result.toString().getBytes();
	}
	
	private byte[] getLaunchConfigContents(final String projectPath) {
		final StringBuilder result = new StringBuilder();
		result.append("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n");
		result.append("<launchConfiguration type=\"org.eclipse.m2e.Maven2LaunchConfigurationType\">\n");
		result.append("<booleanAttribute key=\"M2_DEBUG_OUTPUT\" value=\"false\"/>\n");
		result.append("<stringAttribute key=\"M2_GOALS\" value=\"wtf:generate\"/>\n");
		result.append("<booleanAttribute key=\"M2_NON_RECURSIVE\" value=\"false\"/>\n");
		result.append("<booleanAttribute key=\"M2_OFFLINE\" value=\"false\"/>\n");
		result.append("<stringAttribute key=\"M2_PROFILES\" value=\"\"/>\n");
		result.append("<listAttribute key=\"M2_PROPERTIES\"/>\n");
		result.append("<stringAttribute key=\"M2_RUNTIME\" value=\"EMBEDDED\"/>\n");
		result.append("<booleanAttribute key=\"M2_SKIP_TESTS\" value=\"false\"/>\n");
		result.append("<intAttribute key=\"M2_THREADS\" value=\"1\"/>\n");
		result.append("<booleanAttribute key=\"M2_UPDATE_SNAPSHOTS\" value=\"false\"/>\n");
		result.append("<stringAttribute key=\"M2_USER_SETTINGS\" value=\"\"/>\n");
		result.append("<booleanAttribute key=\"M2_WORKSPACE_RESOLUTION\" value=\"false\"/>\n");
		result.append("<stringAttribute key=\"bad_container_name\" value=\"/PartsList/g\"/>\n");
		result.append("<listAttribute key=\"org.eclipse.debug.ui.favoriteGroups\">\n");
		result.append("<listEntry value=\"org.eclipse.debug.ui.launchGroup.run\"/>\n");
		result.append("</listAttribute>\n");
		result.append("<stringAttribute key=\"org.eclipse.jdt.launching.WORKING_DIRECTORY\" value=\"" + projectPath
				+ "\"/>\n");
		result.append("</launchConfiguration>\n");
		
		return result.toString().getBytes();
	}
	
	private String getOracleDbPropertiesAsString(final Properties properties) {
		final StringBuilder result = new StringBuilder("# Stores Information for connection with Oracle Database\n");
		for (final Entry<Object, Object> e : properties.entrySet()) {
			result.append(e.getKey().toString() + "=" + e.getValue().toString() + "\n");
		}
		return result.toString();
	}
}
